/* libmpdclient
 * (c)2003 by Qball Cow (Qball@Qball.homelinux.org)
 * This project's homepage is: http://www.musicpd.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glade/glade.h>
#include "libmpdclient.h"
#include "eggtrayicon.h"
#include "playlist.h"

/* the tray Icon variables */
EggTrayIcon *tray_icon;
GtkWidget *tray_icon_image;
GtkTooltips *tray_icon_tooltip;

/****************************************
 * The index mouse menu 		*
 ***************************************/

static GtkItemFactoryEntry mouse_menu_items[] =
{
  {"/Play",		"",		play_song,		0, "<StockItem>", "media-play"},
  {"/Pause",		"",		pause_song,		0, "<StockItem>", "media-pause"},
  {"/Resume",		"",		pause_song,		0, "<StockItem>", "media-play"},
  {"/Stop",		"",		stop_song,		0, "<StockItem>", "media-stop"},
  {"/Next",		"",		next_song,		0, "<StockItem>", "media-next"},
  {"/Prev",		"",		prev_song,		0, "<StockItem>", "media-prev"},
  { "/sep",	 	"",	       	0,	      		0, "<Separator>"},
  {"/Playlist Manager",		"",	0,			0, "<ToggleItem>"},
  {"/Player",		"",		0,			0, "<ToggleItem>"},
  { "/sep",	 	"",	       	0,	      		0, "<Separator>"},
  {"/Repeat",		"",		0,			0, "<ToggleItem>"},
  {"/Random",		"",		0,			0, "<ToggleItem>"},
  {"/Volume",		"",		0,			0, "<Branch>"},
  {"/Volume/Volume"	"",		0,			0},
  {"/Volume/Up", 	"",		volume_up,			0, "<StockItem>", GTK_STOCK_GO_UP},	
  {"/Volume/Down", 	"",		volume_down,			0, "<StockItem>", GTK_STOCK_GO_DOWN},	  
  {"/sep",		"",		0,			0, "<Separator>"},
  {"/Quit",		"",		gtk_main_quit,	0, "<StockItem>", GTK_STOCK_QUIT},

};

static int nmouse_menu_items = sizeof (mouse_menu_items) / sizeof (mouse_menu_items[0]);

/* The tray icon functions. */
/* the init function and a internall callback fucnction */


void tray_destroyed(GtkObject *object, gpointer data)
{
  tray_icon = NULL;
	insetting.tray_running = FALSE;
  g_idle_add((GSourceFunc)tray_init, NULL);
	if(!insetting.mw_show) hide_mainw();
}

gboolean tray_create()
    {
    if(insetting.tray_running) return FALSE;
    insetting.tray_running = TRUE;
    return FALSE;
    }


/* create the tray Icon */
gboolean tray_init()
	{
	GtkWidget *eventbox;
	eventbox = gtk_event_box_new ();
        
	/* Creation of the tray icon */
	tray_icon = egg_tray_icon_new ("traympc");
	tray_icon_image = gtk_image_new_from_stock("traymp-tray", GTK_ICON_SIZE_BUTTON);
    
	/* Display the eventbox on the screen */
        gtk_widget_show (eventbox);
										         
	/* Adds the tray icon image onto the event box */
        gtk_container_add (GTK_CONTAINER (eventbox), (GtkWidget *)tray_icon_image);
											       
	/* Adds the eventbox to the tray icon */
        gtk_container_add (GTK_CONTAINER (tray_icon), eventbox);
												 													
	/* Create the tooltip for the tray icon */
	tray_icon_tooltip = gtk_tooltips_new ();

	/* Attaching the tooltip to the tray icon and giving it a text */
	gtk_tooltips_set_tip ((GtkTooltips *)tray_icon_tooltip, GTK_WIDGET (tray_icon), "traympc", NULL);

	/* Show all the widgets on the screen */   
	gtk_widget_show_all (GTK_WIDGET (tray_icon)); 
	g_signal_connect (G_OBJECT (eventbox), "button_press_event", G_CALLBACK (tray_clicked), NULL );
	/* if the widget is destroyed this make sure it's started again (for if the tray apears again) */
    g_signal_connect (G_OBJECT(tray_icon), "destroy",G_CALLBACK (tray_destroyed), NULL);
	g_signal_connect (G_OBJECT(tray_icon_image), "expose-event",  G_CALLBACK (tray_create), NULL);
	return FALSE;	
      }

/* handle the click event on the tray icon */

int tray_clicked(GtkWidget *image, GdkEventButton *event){
	if(!insetting.update_timeout) return 1;
	insetting.update_timeout = FALSE;
	char *buf;
	if(tray_icon == NULL) return 1;
	insetting.update_timeout = TRUE; /* where done.. no need to wait */
		check_error();
 	if((event->type == GDK_BUTTON_PRESS) && (event->button == 3))
        {
	GtkWidget *ms;
	GtkItemFactory *item_factory;

	item_factory = gtk_item_factory_new (GTK_TYPE_MENU, "<popup>", NULL);
	gtk_item_factory_create_items (item_factory, nmouse_menu_items, mouse_menu_items, NULL);
	/* add it */
	ms = gtk_item_factory_get_widget(item_factory, "<popup>");

	if(status->state == MPD_STATUS_STATE_PLAY)
	    {
	    gtk_widget_hide(gtk_item_factory_get_widget(item_factory, "/Play"));
	    gtk_widget_hide(gtk_item_factory_get_widget(item_factory, "/Resume"));
	    }
	else if (status->state == MPD_STATUS_STATE_PAUSE)
	    {
	    gtk_widget_hide(gtk_item_factory_get_widget(item_factory, "/Play"));
	    gtk_widget_hide(gtk_item_factory_get_widget(item_factory, "/Pause"));
	    }
	else{
	    gtk_widget_hide(gtk_item_factory_get_widget(item_factory, "/Resume"));
	    gtk_widget_hide(gtk_item_factory_get_widget(item_factory, "/Pause"));
	    }
	    
	

	
	/* set the repeat toggle in the right mode */
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(item_factory, "/Repeat")), status->repeat);	
	g_signal_connect(G_OBJECT(gtk_item_factory_get_widget(item_factory, "/Repeat")), "toggled", G_CALLBACK(repeat_pl), NULL);

	if(connection->version[1] >= 9)
	    {
	    /* set the random toggle in the right mode */
	    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(item_factory, "/Random")), status->random);	
    	    g_signal_connect(G_OBJECT(gtk_item_factory_get_widget(item_factory, "/Random")), "toggled", G_CALLBACK(random_pl), NULL);
	    }
	else gtk_widget_hide(gtk_item_factory_get_widget(item_factory, "/Random"));

	buf = g_strdup_printf("Volume: %i%%", status->volume);
	gtk_label_set_text(GTK_LABEL(gtk_bin_get_child((GtkBin *)gtk_item_factory_get_widget(item_factory, "/Volume/Volume"))), buf);
	gtk_widget_set_sensitive((GtkWidget *)gtk_item_factory_get_widget(item_factory, "/Volume/Volume"), FALSE);
	
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(item_factory, "/Playlist Manager")), insetting.playlist_running);	
	g_signal_connect(G_OBJECT(gtk_item_factory_get_widget(item_factory, "/Playlist Manager")), "toggled", G_CALLBACK(show_playlist_manager), NULL);

	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(item_factory, "/Player")), insetting.mw_show);	
	if(!insetting.tray_running) gtk_widget_set_sensitive((GtkWidget *)gtk_item_factory_get_widget(item_factory, "/Player"), FALSE);
	g_signal_connect(G_OBJECT(gtk_item_factory_get_widget(item_factory, "/Player")), "toggled", G_CALLBACK(hide_mainw), NULL);



	g_free(buf);
	gtk_menu_popup(GTK_MENU(ms), NULL, NULL, NULL, NULL, event->button, event->time);

	}
    else if((event->type == GDK_BUTTON_PRESS) && (event->button == 1) && 
	    (status->state == MPD_STATUS_STATE_PLAY || status->state == MPD_STATUS_STATE_PAUSE)) pause_song();
    else if((event->type == GDK_BUTTON_PRESS) && (event->button == 2) && 
	    status->state == MPD_STATUS_STATE_PLAY) next_song();
    else if((event->type == GDK_2BUTTON_PRESS) && (event->button == 1)) hide_mainw();
    return 1;
}
